@@ -1,4 +1,5 @@
 (function() {
+  var active = null;
   var obj = null;
   var tid = 0;
   var qs = '';
@@ -50,6 +51,7 @@
       };
 
       document.getElementById('print').onclick = function() {
+        sobj.innerHTML = '';
         var t = document.getElementById('t_' + document.querySelector('.tab-content > .active').getAttribute('id'));
 
         if (t.nodeName == 'IFRAME') {
@@ -61,12 +63,60 @@
         }
       };
 
+      document.getElementById('docx').onclick = function() {
+        sobj.innerHTML = '';
+        if (obj != null) {
+          var xHR = new XMLHttpRequest();
+          xHR.open("POST", 'html2docx' + qs, true);
+
+          xHR.onload = function() {
+            if (this.status === 200) {
+              var link = document.createElement('a');
+              link.href = window.URL.createObjectURL(xHR.response);
+              link.download = xHR.getResponseHeader("X-Download-Filename");
+              link.click();
+            }
+            else {
+              var sT = (this.statusText.length == 0) ? window.opener.getStatusText(this.status) : this.statusText;
+              set_status("darkred", "HTTP ERROR " + this.status, sT);
+            }
+          };
+
+          xHR.timeout = 10000;
+          xHR.onerror = function() {
+            set_status("darkred", "ERROR", "XMLHttpRequest.onError()");
+          };
+          xHR.ontimeout = function() {
+            set_status("darkred", "ERROR", "XMLHttpRequest.onTimeout()");
+          };
+
+          xHR.responseType = "blob";
+          xHR.setRequestHeader("Content-Type", "application/json");
+
+          var o = JSON.stringify(obj.outputs[active]);
+          if (o.length > 1024) {
+            xHR.setRequestHeader("Content-Encoding", "gzip");
+            xHR.send(pako.gzip(o));
+          }
+          else {
+            xHR.send(o);
+          }
+        }
+        var t = document.getElementById('t_' + document.querySelector('.tab-content > .active').getAttribute('id'));
+        t.focus();
+      };
+
       document.getElementById('download').onclick = function() {
         sobj.innerHTML = '';
         if (obj != null) {
           var zip = new JSZip();
           Object.keys(obj.outputs).forEach(function(o) {
-            var [ofile, oformat] = o.split(':', 2);
+            var ofile = o.replace(/^[0-9]+\//, '');
+            var oformat = 'text';
+
+            if (ofile.includes(':')) {
+              [ofile, oformat] = ofile.split(':', 2);
+            }
 
             ofile = ofile.replace(/[^A-Z0-9_. -/]/gi, '_').replace(/_+/g, '_');
 
@@ -124,25 +174,68 @@
 
                 var links = '';
                 var tabs = '';
+                var section = undefined;
+                var dflag = false;
+                var sort_keys = {};
 
+                for (var [k, v] of Object.entries(obj.outputs)) {
+                  if (!k.includes('/')) {
+                    var o = k.substring(0, k.lastIndexOf(':'));
+
+                    obj.outputs['0/' + k] = v;
+                    delete (obj.outputs[k]);
+
+                    if (sort_keys.hasOwnProperty(o)) {
+                      if (sort_keys[o] != '0') {
+                        document.title = "Error";
+                        document.body.innerHTML = "<div id=\"status\" class=\"alert alert-danger\"><strong><h4>JinjaFx Error</h4></strong>Duplicate Outputs with Different Numerical Sort Keys is Prohibited</div>";
+                        return;
+                      }
+                    }
+                    else {
+                      sort_keys[o] = '0';
+                    }
+                  }
+                  else {
+                    var m = k.match(/^([0-9]+)\/(.+):\S+/);
+                    if (m) {
+                      if (sort_keys.hasOwnProperty(m[2])) {
+                        if (sort_keys[m[2]] != m[1]) {
+                          document.title = "Error";
+                          document.body.innerHTML = "<div id=\"status\" class=\"alert alert-danger\"><strong><h4>JinjaFx Error</h4></strong>Duplicate Outputs with Different Numerical Sort Keys is Prohibited</div>";
+                          return;
+                        }
+                      }
+                      else {
+                        sort_keys[m[2]] = m[1];
+                      }
+                    }
+                    if (k.replace(/^[0-9]+\//, '').includes('/')) {
+                      dflag = true;
+                    }
+                  }
+                }
+                
                 Object.keys(obj.outputs).sort(function(a, b) {
-                  if (a == 'Output') {
-                    return -1;
-                  }
+                  a = a.split(':').slice(0, -1).join(':')
+                  b = b.split(':').slice(0, -1).join(':')
                   return a > b ? 1 : b > a ? -1 : 0;
+
                 }).forEach(function(output) {
-                  var oname = output;
-                  var oformat = 'text';
-
-                  if (output.includes(':')) {
-                    oname = output.substring(0, output.lastIndexOf(':'));
-                    oformat = output.substring(output.lastIndexOf(':') + 1);
-                  }
+                  var oname = output.substring(0, output.lastIndexOf(':'));
+                  oname = oname.replace(/^[0-9]+\//, '');
 
+                  var oformat = output.substring(output.lastIndexOf(':') + 1);
                   var g = window.opener.quote(oname)
+                  var e = g.split(/\/+/);
 
                   tabs += '<div id="o' + oid + '" class="h-100 tab-pane fade' + ((oid == 1) ? ' show active' : '') + '">';
-                  tabs += '<h4 class="fw-bold">' + g + '</h4>';
+                  if (dflag) {
+                    tabs += '<h4 class="fw-bold">' + e[e.length - 1] + '</h4>';
+                  }
+                  else {
+                    tabs += '<h4 class="fw-bold">' + g + '</h4>';
+                  }
 
                   var tc = window.opener.d(obj.outputs[output]);
                   if (oformat == 'html') {
@@ -154,10 +247,26 @@
 
                   tabs += '</div>';
 
-                  links += '<li class="nav-item">';
-                  links += '<a class="nav-link' + ((oid == 1) ? ' active"' : '"') + ' data-bs-toggle="tab" href="#o' + oid + '">' + g + '</a>';
-                  links += '</li>';
-
+                  if (dflag) {
+                    var dir = e.slice(0, -1).join('/');
+                    if (section != dir) {
+                      if (section == undefined) {
+                        links += '<div class="directory">' + dir + '/</div>'
+                      }
+                      else {
+                        links += '<div class="mt-3 directory">' + dir + '/</div>'
+                      }
+                      section = dir;
+                    }
+                    links += '<li class="nav-item">';
+                    links += '<a class="nav-link' + ((oid == 1) ? ' active"' : '"') + ' data-id="' + output + '" data-format="' + oformat + '" data-bs-toggle="tab" href="#o' + oid + '">' + e[e.length - 1] + '</a>';
+                    links += '</li>';
+                  }
+                  else {
+                    links += '<li class="nav-item">';
+                    links += '<a class="nav-link' + ((oid == 1) ? ' active"' : '"') + ' data-id="' + output + '" data-format="' + oformat + '" data-bs-toggle="tab" href="#o' + oid + '">' + g + '</a>';
+                    links += '</li>';
+                  }
                   oid += 1;
                 });
 
@@ -182,6 +291,26 @@
                 window.onresize();
                 document.body.style.display = 'block';
 
+                function toggle_docx() {
+                  var e = document.getElementsByClassName('nav-link active');
+                  for (var i = 0; i < e.length; i++) {
+                    active = e.item(i).getAttribute('data-id');
+                    if (e.item(i).getAttribute('data-format') == "html") {
+                      document.getElementById('docx').classList.remove('d-none');
+                    }
+                    else {
+                      document.getElementById('docx').classList.add('d-none');
+                    }
+                  }
+                }
+
+                var e = document.getElementsByClassName('nav-link');
+                for (var i = 0; i < e.length; i++) {
+                  e.item(i).onclick = toggle_docx;
+                }
+
+                toggle_docx();
+
                 if (stderr != null) {
                   var html = '<ul class="mb-0">'
                   stderr.trim().split(/\n+/).forEach(function(w) {
@@ -200,13 +329,13 @@
               }
               else {
                 document.title = "Error";
-                document.body.innerHTML = "<div id=\"status\" class=\"alert alert-danger\"><strong><h4>JinjaFx Error</h4></strong><pre>"+ obj.error + "</pre></div>";
+                document.body.innerHTML = "<div id=\"status\" class=\"alert alert-danger\"><strong><h4>JinjaFx Error</h4></strong><pre>"+ window.opener.quote(obj.error) + "</pre></div>";
               }
             }
             catch (e) {
               console.log(e);
               document.title = "Error";
-              document.body.innerHTML = "<div id=\"status\" class=\"alert alert-danger\"><strong><h4>Internal Error</h4></strong>" + e + "</div>";
+              document.body.innerHTML = "<div id=\"status\" class=\"alert alert-danger\"><strong><h4>Internal Error</h4></strong>" + window.opener.quote(e) + "</div>";
             }
           }
           else {
